import { Filter, Paging, Query, SortField } from '@nestjs-query/core';
import { FindOptions, Filterable, DestroyOptions, Order, UpdateOptions, CountOptions } from 'sequelize';
import { WhereBuilder } from './where.builder';
/**
 * @internal
 *
 * Interface that for `sequelize` query builders that are sortable.
 */
interface Sortable<Entity> {
    order?: Order;
}
/**
 * @internal
 *
 * Interface for `sequelize` query builders that are pageable.
 */
interface Pageable<Entity> {
    limit?: number;
    offset?: number;
}
/**
 * @internal
 *
 * Class that will convert a Query into a `sequelize` Query Builder.
 */
export declare class FilterQueryBuilder<Entity> {
    readonly whereBuilder: WhereBuilder<Entity>;
    constructor(whereBuilder?: WhereBuilder<Entity>);
    /**
     * Create a `sequelize` SelectQueryBuilder with `WHERE`, `ORDER BY` and `LIMIT/OFFSET` clauses.
     *
     * @param query - the query to apply.
     */
    findOptions(query: Query<Entity>): FindOptions;
    countOptions(query: Query<Entity>): CountOptions;
    /**
     * Create a `sequelize` DeleteQueryBuilder with a WHERE clause.
     *
     * @param query - the query to apply.
     */
    destroyOptions(query: Query<Entity>): DestroyOptions;
    /**
     * Create a `sequelize` UpdateQueryBuilder with `WHERE` and `ORDER BY` clauses
     *
     * @param query - the query to apply.
     */
    updateOptions(query: Query<Entity>): UpdateOptions;
    /**
     * Applies paging to a Pageable `sequelize` query builder
     * @param qb - the `sequelize` QueryBuilder
     * @param paging - the Paging options.
     */
    applyPaging<P extends Pageable<Entity>>(qb: P, paging?: Paging): P;
    /**
     * Applies the filter from a Query to a `sequelize` QueryBuilder.
     *
     * @param qb - the `sequelize` QueryBuilder.
     * @param filter - the filter.
     */
    applyFilter<Where extends Filterable>(filterable: Where, filter?: Filter<Entity>): Where;
    /**
     * Applies the ORDER BY clause to a `sequelize` QueryBuilder.
     * @param qb - the `sequelize` QueryBuilder.
     * @param sorts - an array of SortFields to create the ORDER BY clause.
     */
    applySorting<T extends Sortable<Entity>>(qb: T, sorts?: SortField<Entity>[]): T;
}
export {};
