"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelationQueryService = void 0;
const core_1 = require("@nestjs-query/core");
const query_1 = require("../query");
/**
 * Base class to house relations loading.
 * @internal
 */
class RelationQueryService {
    async queryRelations(RelationClass, relationName, dto, query) {
        if (Array.isArray(dto)) {
            return this.batchQueryRelations(RelationClass, relationName, dto, query);
        }
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationQueryBuilder = this.getRelationQueryBuilder();
        const relations = await this.ensureIsEntity(dto).$get(relationName, relationQueryBuilder.findOptions(assembler.convertQuery(query)));
        return assembler.convertToDTOs(relations);
    }
    async countRelations(RelationClass, relationName, dto, filter) {
        if (Array.isArray(dto)) {
            return this.batchCountRelations(RelationClass, relationName, dto, filter);
        }
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationQueryBuilder = this.getRelationQueryBuilder();
        return this.ensureIsEntity(dto).$count(relationName, relationQueryBuilder.countOptions(assembler.convertQuery({ filter })));
    }
    async findRelation(RelationClass, relationName, dto) {
        if (Array.isArray(dto)) {
            return this.batchFindRelations(RelationClass, relationName, dto);
        }
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relation = await this.ensureIsEntity(dto).$get(relationName);
        if (!relation) {
            return undefined;
        }
        return assembler.convertToDTO(relation);
    }
    /**
     * Add a single relation.
     * @param id - The id of the entity to add the relation to.
     * @param relationName - The name of the relation to query for.
     * @param relationIds - The ids of relations to add.
     */
    async addRelations(relationName, id, relationIds) {
        const entity = await this.getById(id);
        await entity.$add(relationName, relationIds);
        return entity;
    }
    /**
     * Set the relation on the entity.
     *
     * @param id - The id of the entity to set the relation on.
     * @param relationName - The name of the relation to query for.
     * @param relationId - The id of the relation to set on the entity.
     */
    async setRelation(relationName, id, relationId) {
        const entity = await this.getById(id);
        await entity.$set(relationName, relationId);
        return entity;
    }
    /**
     * Removes multiple relations.
     * @param id - The id of the entity to add the relation to.
     * @param relationName - The name of the relation to query for.
     * @param relationIds - The ids of the relations to add.
     */
    async removeRelations(relationName, id, relationIds) {
        const entity = await this.getById(id);
        await entity.$remove(relationName, relationIds);
        return entity;
    }
    /**
     * Remove the relation on the entity.
     *
     * @param id - The id of the entity to set the relation on.
     * @param relationName - The name of the relation to query for.
     * @param relationId - The id of the relation to set on the entity.
     */
    async removeRelation(relationName, id, relationId) {
        const entity = await this.getById(id);
        const association = this.getAssociation(relationName);
        if (association.isSingleAssociation) {
            // todo update that this line to remove the casting once https://github.com/RobinBuschmann/sequelize-typescript/issues/803 is addressed.
            await entity.$set(relationName, null);
        }
        else {
            await entity.$remove(relationName, relationId);
        }
        return entity;
    }
    getRelationQueryBuilder() {
        return new query_1.FilterQueryBuilder();
    }
    /**
     * Query for an array of relations for multiple dtos.
     * @param RelationClass - The class to serialize the relations into.
     * @param entities - The entities to query relations for.
     * @param relationName - The name of relation to query for.
     * @param query - A query to filter, page or sort relations.
     */
    async batchQueryRelations(RelationClass, relationName, entities, query) {
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationQueryBuilder = this.getRelationQueryBuilder();
        const findOptions = relationQueryBuilder.findOptions(assembler.convertQuery(query));
        return entities.reduce(async (mapPromise, e) => {
            const map = await mapPromise;
            const relations = await this.ensureIsEntity(e).$get(relationName, findOptions);
            map.set(e, assembler.convertToDTOs(relations));
            return map;
        }, Promise.resolve(new Map()));
    }
    /**
     * Query for an array of relations for multiple dtos.
     * @param RelationClass - The class to serialize the relations into.
     * @param entities - The entities to query relations for.
     * @param relationName - The name of relation to query for.
     * @param query - A query to filter, page or sort relations.
     */
    async batchCountRelations(RelationClass, relationName, entities, filter) {
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationQueryBuilder = this.getRelationQueryBuilder();
        const findOptions = relationQueryBuilder.countOptions(assembler.convertQuery({ filter }));
        return entities.reduce(async (mapPromise, e) => {
            const map = await mapPromise;
            const count = await this.ensureIsEntity(e).$count(relationName, findOptions);
            map.set(e, count);
            return map;
        }, Promise.resolve(new Map()));
    }
    /**
     * Query for a relation for multiple dtos.
     * @param RelationClass - The class to serialize the relations into.
     * @param dtos - The dto to query relations for.
     * @param relationName - The name of relation to query for.
     * @param query - A query to filter, page or sort relations.
     */
    async batchFindRelations(RelationClass, relationName, dtos) {
        const assembler = core_1.AssemblerFactory.getAssembler(RelationClass, this.getRelationEntity(relationName));
        return dtos.reduce(async (mapPromise, e) => {
            const map = await mapPromise;
            const relation = await this.ensureIsEntity(e).$get(relationName);
            if (relation) {
                map.set(e, assembler.convertToDTO(relation));
            }
            return map;
        }, Promise.resolve(new Map()));
    }
    ensureIsEntity(e) {
        if (!(e instanceof this.model)) {
            return this.model.build(e);
        }
        return e;
    }
    getAssociation(relationName) {
        const association = this.model.associations[relationName];
        if (!association) {
            throw new Error(`Unable to find relation ${relationName} on ${this.model.name}`);
        }
        return association;
    }
    getRelationEntity(relationName) {
        return this.getAssociation(relationName).target;
    }
}
exports.RelationQueryService = RelationQueryService;
//# sourceMappingURL=relation-query.service.js.map